require 'upkey/helios/accessor'
require 'upkey/helios/builder'
require 'upkey/helios/responder'
require 'upkey/helios/response'

require 'upkey/helios/errors/missing_argument_error'
require 'upkey/helios/errors/helios_response_error'

require 'faraday'

module Upkey
  module Helios
    class Client
      attr_reader :access_key, :helios_url, :conn

      def initialize(args = {})
        raise MissingArgumentError unless args[:access_key] && args[:helios_url]

        @access_key = args[:access_key]
        @helios_url = args[:helios_url]
        @conn = set_up_faraday
      end

      def method_missing(*args)
        args[2].nil? ? (params = {}) : (params = args[2])
        response = Upkey::Helios::Accessor.query(self, args[0], args[1], params)

        Upkey::Helios::Responder.parse(response)
      end

      private

      def set_up_faraday
        Faraday.new(:url => "#{@helios_url}") do |faraday|
          faraday.request  :url_encoded
          faraday.response :logger
          faraday.adapter  Faraday.default_adapter
        end
      end
    end
  end
end
